import logging
import os
import re

_right_padding = 20
color_ctrl_re = re.compile("ℂ([0-9]*)\\.")

log = logging.getLogger("sort-images")


def _trim_to_terminal_width(string: str):
    string = str(string)
    try:
        terminal_width = os.get_terminal_size().columns - _right_padding
    except OSError:
        return string

    if len(string) <= terminal_width:
        return string

    return string[:terminal_width // 2 - 3] + "..." + string[len(string) - terminal_width // 2:]


def color_start(color):
    return f"\x1b[38;5;{color}m"


def color_end():
    return f"\x1b[m"


def parse_color(string: str) -> str:
    """
    Replaces strings of the form ℂ([0-9]*)\\. with a console control code
    that colors the following text (and removes color if the thing in the brackets is empty)"""

    output = ""
    last_match_index = 0

    for m in color_ctrl_re.finditer(string):
        output += string[last_match_index:m.start()]

        if m.group(1) == "":
            output += color_end()
        else:
            output += color_start(m.group(1))

        last_match_index = m.end()

    return output + string[last_match_index:]


def print_temp(string):
    """Print a temporary string to the console"""
    hide_temp()
    print(_trim_to_terminal_width(string), end="", flush=True)


def hide_temp():
    """Remove the current temporary string from the console"""
    print("\x1b[1G\x1b[2K", end="", flush=True)



import argparse
import logging
from pathlib import Path

from sort_images.terminal_formatting import parse_color
from sort_images import sort
from sort_images.version import program_version

log = logging.getLogger("sort-images")
console = logging.StreamHandler()
log.addHandler(console)
log.setLevel(logging.DEBUG)
console.setFormatter(
    logging.Formatter(parse_color("{asctime} [ℂ3.{levelname:>5}ℂ.] ℂ4.{name}ℂ.: {message}"),
                      style="{", datefmt="%W %a %I:%M"))

PROGRAM_NAME = "sort-images"
DEFAULT_DATE_FORMAT = "%y/%b/%NAME"


def command_entry_point():
    try:
        main()
    except KeyboardInterrupt:
        log.warning("Program was interrupted by user")


def main():
    parser = argparse.ArgumentParser(prog=PROGRAM_NAME,
                                     description="A cli tool to sort images into folders by date",
                                     allow_abbrev=True, add_help=True, exit_on_error=True)

    parser.add_argument("-f", "--format",
                        help=f"Set the output date format, the default is {DEFAULT_DATE_FORMAT.replace("%", "%%")} "
                             f"(The format is the standard python datetime format, except for the special variable %%NAME that allows "
                             f"for the insertion of the file name)", default=DEFAULT_DATE_FORMAT)
    parser.add_argument("-d", "--dry-run", action="store_true", help="Don't copy any files")
    parser.add_argument("-m", "--move", action="store_true", help="Move files instead of copying them")
    parser.add_argument('-v', '--verbose', action='store_true', help="Show more output")
    parser.add_argument("--version", action="store_true", help="Show the current version of the program")
    parser.add_argument("SRC", help=f"The folder in which {PROGRAM_NAME} will look for images")
    parser.add_argument("DST",
                        help="The folder into which the sorted images will be placed (possibly with a path prepended)")

    args = parser.parse_args()

    log.setLevel(logging.DEBUG if args.verbose else logging.INFO)
    log.debug("Starting program...")

    if args.version:
        log.info(f"{PROGRAM_NAME} version {program_version}")
        return

    src = Path(args.SRC)
    dst = Path(args.DST)
    frmt = args.format

    if not src.exists():
        log.error(f"Folder {src} does not exist")
        return

    log.debug("Now starting to sort")
    sort.sort(src, dst, frmt, dry_run=args.dry_run, move=args.move)
